function bbso_gs,im,kernel_size,selection,imsize

;+
; NAME:
;        BBSO_GS
;
; PURPOSE:
;        Convolution with gaussian
;
; CALLING SEQUENCE:
;        Result = BBSO_GS(image,kernel_size,selection,imsize)
;
; INPUTS:
;        im          = input image
;
;        kernel_size = size of smoothing kernel
;
;        selection   = selected image area (display area coordinates)
;
;        imsize      = size of currend display area     
;     
; OUTPUTS:
;        Smoothed image
;
; SIDE EFFECTS:
;        None
;
; COMMON BLOCKS:
;        None.
;
; RESTRICTIONS:
;        None.
;
; PROCEDURE:
;        Standard convolution (Gaussian kernel)
;
; MODIFICATION HISTORY:
;        Written by Anders Johannesson, April 1995  as ajsmooth
;        Modified by Anders Johannesson, August 1996
;
;-

ON_ERROR,2

;# CHECK KERNEL SIZE
if (kernel_size mod 2) eq 0 then begin
   kernel_size=kernel_size+1
end

;# MAKE KERNEL
kernel=indgen(kernel_size,kernel_size)
yy=kernel/kernel_size
xx=kernel - yy*kernel_size
yy=yy-kernel_size/2
xx=xx-kernel_size/2
efold=float(kernel_size/2)
kernel=fix(fix(255)*exp(-(xx^2+yy^2)/efold^2))

;# CONVOLUTION
fim=convol(im,kernel,/EDGE_TRUNCATE,total(kernel))

;# WHAT IS THE CURRENT SELECTION?
dim=size(im)
x1=selection(2)
x2=selection(5)
y1=selection(7)-(imsize(1)-dim(2))
y2=selection(8)-(imsize(1)-dim(2))
xmax=dim(1)-1
ymax=dim(2)-1

;# IS A MARQUEE SELECTION ACTIVE?
if (x1 gt 0) and (y1 gt 0) and (x2 lt xmax) and (y2 lt ymax) then begin

   ;# IS THE SELECTION INVERTED?
   if selection(1) eq 0 then begin
      out=im
      out(x1,y1)=fim(x1:x2,y1:y2)
   endif else begin
      out=fim
      out(x1,y1)=im(x1:x2,y1:y2)
   endelse

endif else begin
   out=fim
endelse

return,out   

end

