pro bbsoflat
;# BBSOFITS ROUTINE FOR FLATFIELDING
;# exposure check (expcheck)  1-enabled  0-disabled
;# Scot Kleinman 1996

COMMON IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$
       outname, helpfile, selection,oselection,traceplot,saveflag

    WIDGET_CONTROL, /HOURGLASS
    nd=1 ; flag: 1= no dark correction for flat; 0 = dark correction
    ;# PICK FLAT FILE IN DIFFERENT OS
    a=!version & ver_no=fix(strmid(a.release,0,1))
    if ver_no gt 3 then begin
       if strpos(!version.os_family,'Windows') ne -1 then begin
          ffilename=pickfile(GET_PATH=newpath,FILTER=['*.fts','*.fit','*.jpg'],$
                            TITLE='Select Flat Image')
       endif else begin
          ffilename=pickfile(GET_PATH=newpath,FILTER = ['*.fts *.fit *.jpg'],$
                            TITLE = 'Select Flat Image')
       endelse
    endif else begin
       ffilename=pickfile(GET_PATH=newpath,FILTER = ['*.fts','*.fit','*.jpg'],$
                         TITLE = 'Select Flat Image')
    endelse
    str_start=strlen(newpath)
    str_length=strlen(ffilename)-str_start
    ffilename=strmid(ffilename,str_start,str_length)
    ffilename=strlowcase(ffilename)
    if strlen(ffilename) gt 1 then begin
       WIDGET_CONTROL, /HOURGLASS
       cd,newpath
       extension=strmid(ffilename,rstrpos(ffilename,'.')+1,3)
       if (extension eq 'fts') or (extension eq 'fit') then begin
          fl=readfits(ffilename,flhed,/SILENT,/NOSCALE)
       endif
       if (extension eq 'jpg') then begin
          read_jpeg,ffilename,fl
          ;# IF COLOR JPEG USE ONLY GREEN CHANNEL ###
          flhed='NO HEADER - image was read as JPEG'
       endif
    fdim=size(fl)

    ;# PICK DARK FILE FOR FLAT IN DIFFERENT OS
    a=!version & ver_no=fix(strmid(a.release,0,1))
    if ver_no gt 3 then begin
       if strpos(!version.os_family,'Windows') ne -1 then begin
          dfilename=pickfile(GET_PATH=newpath,FILTER=['*.fts','*.fit','*.jpg'],$
                            TITLE='Select Dark for the Flat Image')
       endif else begin
          dfilename=pickfile(GET_PATH=newpath,FILTER = ['*.fts *.fit *.jpg'],$
                            TITLE = 'Select Dark for the Flat Image')
       endelse
    endif else begin
       dfilename=pickfile(GET_PATH=newpath,FILTER = ['*.fts','*.fit','*.jpg'],$
                         TITLE = 'Select Dark for the Flat Image')
    endelse
    str_start=strlen(newpath)
    str_length=strlen(dfilename)-str_start
    dfilename=strmid(dfilename,str_start,str_length)
    dfilename=strlowcase(dfilename)
    if strlen(dfilename) gt 1 then begin
       WIDGET_CONTROL, /HOURGLASS
       cd,newpath
       extension=strmid(dfilename,rstrpos(dfilename,'.')+1,3)
       if (extension eq 'fts') or (extension eq 'fit') then begin
          dk=readfits(dfilename,dkhed,/SILENT,/NOSCALE)
       endif
       if (extension eq 'jpg') then begin
          read_jpeg,dfilename,dk
          ;# IF COLOR JPEG USE ONLY GREEN CHANNEL ###
          dkhed='NO HEADER - image was read as JPEG'
       endif

       ; Check integration times and image dimensions first
         ddim=size(dk)

       if (extension eq 'jpg') then begin

          ;# MESSAGE
          a=!version & ver_no=fix(strmid(a.release,0,1))
          if ver_no gt 3 then begin
             r=WIDGET_MESSAGE('Dark image is JPG'+STRING(13B)+$
                           'Cannot check exposure times.')
          endif else begin
             mess='Dark image is JPEG - cannot check exposure time'
             ierr=execute('bbsomess,mess')
          endelse

          fexp=0
          dexp=0   ; user already warned- so pass the check below
       endif else begin  ;dark file is fits- so get value from header
          if (strpos(flhed(0),'NO HEADER') ne 0) then begin ;flat is FITS
              fexp=sxpar(flhed,"EXPOSURE")
              dexp=sxpar(dkhed,"EXPOSURE")
          endif else begin

              ;# MESSAGE
              a=!version & ver_no=fix(strmid(a.release,0,1))
              if ver_no gt 3 then begin
                 r=WIDGET_MESSAGE('Flat image is JPG'+STRING(13B)+$
                               'Cannot check exposure times.')
              endif else begin
                 mess='Flat image is JPEG - cannot check exposure time'
                 ierr=execute('bbsomess,mess')
              endelse

              fexp=0
              dexp=0   ; user already warned- so pass the check below
          endelse
       endelse

       if ( (fdim(1) NE ddim(1)) OR (fdim(2) NE ddim(2)) ) then begin

          ;# MESSAGE
          a=!version & ver_no=fix(strmid(a.release,0,1))
          if ver_no gt 3 then begin
          r=WIDGET_MESSAGE('Dark & Flat frames are not the same size.'+$
                           STRING(13B)+'Will not use Dark in Flat correction.')
          endif else begin
             mess='Dark & Flat different size. Dark excluded.'
             ierr=execute('bbsomess,mess')
          endelse

       endif else begin
          if (dexp NE fexp) then begin

             ;# Warn user about dark and flat exposure times
             a=!version & ver_no=fix(strmid(a.release,0,1))
             if ver_no gt 3 then begin
             r=WIDGET_MESSAGE('Dark & Flat Frame exposure times do not match'+$
                              STRING(13B)+$
                              'Will not use Dark in Flat correction.')
             endif else begin
                mess='Dark & Flat have different exposures. Dark excluded.'
                ierr=execute('bbsomess,mess')
             endelse

          endif else begin
             fl=fl-dk
             nd=0
          endelse
       endelse
    endif else begin

       ;# warn user no dark image was chosen for flat
       a=!version & ver_no=fix(strmid(a.release,0,1))
       if ver_no gt 3 then begin
       r=WIDGET_MESSAGE('No Dark Image chosen for'+STRING(13B)+$
                        'The Flat Field.'+STRING(13B)+'Will not subtract any'+$
                        STRING(13B)+'Dark from '+ffilename+'.')
       endif else begin
          mess='No Dark image selected.'
          ierr=execute('bbsomess,mess')
       endelse

    endelse

       ;Check Flat and Image dimensions
       if ( (fdim(1) NE dim(1)) OR (fdim(2) NE dim(2)) ) then begin

          ;# MESSAGE
          a=!version & ver_no=fix(strmid(a.release,0,1))
          if ver_no gt 3 then begin
              r=WIDGET_MESSAGE('Flat Frame & Image are not the same size.')
          endif else begin
             mess='Flat Frame & Image are not the same size.'
             ierr=execute('bbsomess,mess')
          endelse

       endif else begin
          imlast=im
          ocontrast=contrast
          oselection=selection
          ; Do something about zeros!
          index=where(fl eq 0, count)
          if (count ne 0) then fl(index)=1
          ;# APPLY FLATFIELD (KEEP LEVEL OF IMAGE)
          WIDGET_CONTROL, /HOURGLASS
          part=fl(9*fdim(1)/20:11*fdim(1)/20,9*fdim(2)/20:11*fdim(2)/20)
          dum=stdev(part,flevel)
          im=fix(round(float(im)/float(fl)*flevel))
          ;# RESET SELECTION
          p1=[0.0,0.0+(imsize(1)-dim(2))]
          p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))]
          selection=fill_select(0.0,0.0,p1,p2,p1)
          ;# MAKE BYTE SHORTWORD
          if dim(3) eq 1 then im=fix(im)
          pim=im(40:dim(1)-41,40:dim(2)-41)
          contrast=[long(min(pim)),long(max(pim))]
          erase
          tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2)
          loadct,0
          ; add header history line

          if (strpos(header(0),'NO HEADER') ne 0) then begin ;image is FITS
             oheader=header
             hl='Flat Correction with ' + ffilename
             sxaddhist,hl,header
             if (nd eq 0) then begin
                hl='Flat was dark corrected with ' + dfilename
                sxaddhist,hl,header
             endif
          endif

       endelse
    endif

end
