  pro bbsosubt,intitle,expcheck
  ;# BBSOFITS ROUTINE FOR IMAGE SUBTRACTION
  ;# exposure check (expcheck)  1-enabled  0-disabled
  ;# Scot Kleinman 1996

   COMMON IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$
         outname, helpfile, selection,oselection,traceplot,saveflag

   COMMON IMAGETEXT_Comm, IMAGETEXT

    WIDGET_CONTROL, /HOURGLASS
    ;# PICK DARK FILE IN DIFFERENT OS
    a=!version & ver_no=fix(strmid(a.release,0,1))
    if ver_no gt 3 then begin
       if strpos(!version.os_family,'Windows') ne -1 then begin
          dfilename=pickfile(GET_PATH=newpath,FILTER=['*.fts','*.fit','*.jpg'],$
                            TITLE= intitle)
       endif else begin
          dfilename=pickfile(GET_PATH=newpath,FILTER = ['*.fts *.fit *.jpg'],$
                            TITLE = intitle)
       endelse
    endif else begin
       dfilename=pickfile(GET_PATH=newpath,FILTER = ['*.fts','*.fit','*.jpg'],$
                         TITLE = intitle)
    endelse
    str_start=strlen(newpath)
    str_length=strlen(dfilename)-str_start
    dfilename=strmid(dfilename,str_start,str_length)
    dfilename=strlowcase(dfilename)
    if strlen(dfilename) gt 1 then begin
       WIDGET_CONTROL, /HOURGLASS
       cd,newpath
       extension=strmid(dfilename,rstrpos(dfilename,'.')+1,3)
       if (extension eq 'fts') or (extension eq 'fit') then begin
          dk=readfits(dfilename,dkhed,/SILENT,/NOSCALE)
       endif
       if (extension eq 'jpg') then begin
          read_jpeg,dfilename,dk
          ;# IF COLOR JPEG USE ONLY GREEN CHANNEL ###
          dkhed='NO HEADER - image was read as JPEG'
       endif

       ;##################################################################
       ; Check exposure and size (exposure only if enabled)
       ;##################################################################

       ddim=size(dk)
       if expcheck eq 1 then begin  ;expcheck=1 for dark, 0 for subtraction
          if (extension eq 'jpg')  then begin

             ;# warn user if checking exposure
             a=!version & ver_no=fix(strmid(a.release,0,1))
             if ver_no gt 3 then begin
                r=WIDGET_MESSAGE('The Dark image is JPG'+STRING(13B)+$
                                 'Cannot check exposure time.')
             endif else begin
                mess='Dark image is JPG - Cannot check exposure time.'
                ierr=execute('bbsomess,mess')
             endelse

             iexp=0
             dexp=0   ; user already warned- so pass the check below

          endif else begin  ; dark file is fits see if image is fits

             if (strpos(header(0),'NO HEADER') ne 0) then begin ;image is FITS
                 iexp=sxpar(header,"EXPOSURE")
                 dexp=sxpar(dkhed,"EXPOSURE")
             endif else begin

               ;issue warning since image is jpg - cannot check exp times
               a=!version & ver_no=fix(strmid(a.release,0,1))
               if ver_no gt 3 then begin
                  r=WIDGET_MESSAGE('Image is JPG'+STRING(13B)+$
                                   'Cannot check exposure time.')
               endif else begin
                  mess='Image is JPG - Cannot check exposure time.'
                  ierr=execute('bbsomess,mess')
               endelse

               iexp=0
               dexp=0   ; user already warned- so pass the check below
             endelse
          endelse

       endif else begin ;exposure check is off (expcheck=0)
          iexp=0 ; so just set these values so the test below is passed
          dexp=0
       endelse

       ;Now check dimensions of images
       if ( (dim(1) ne ddim(1)) OR (dim(2) ne ddim(2)) ) then begin

          ;# Warn user images are different sizes
          a=!version & ver_no=fix(strmid(a.release,0,1))
          if ver_no gt 3 then begin
             r=WIDGET_MESSAGE('Images are not the same size!')
          endif else begin
             mess='Images are not the same size'
             ierr=execute('bbsomess,mess')
          endelse

       endif else begin

          if (dexp NE iexp) then begin

             ;# Give error message if exposure times don't match
             a=!version & ver_no=fix(strmid(a.release,0,1))
             if ver_no gt 3 then begin
                r=WIDGET_MESSAGE('Exposure times do not match')
             endif else begin
                mess='Exposure times do not match'
                ierr=execute('bbsomess,mess')
             endelse

          endif else begin
             imlast=im
             ocontrast=contrast
             oselection=selection
             im=im-dk
             ; add header history line to header
             if (strpos(header(0),'NO HEADER') ne 0) then begin ;image is FITS
                oheader=header
                if expcheck eq 0 then begin ;subtraction operation
                   hl='Subtraction with ' + dfilename
                endif else begin
                   hl='Dark subtraction with ' + dfilename
                endelse
                sxaddhist,hl,header
             endif

             dim=size(im)
             ;# RESET SELECTION
             p1=[0.0,0.0+(imsize(1)-dim(2))]
             p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))]
             selection=fill_select(0.0,0.0,p1,p2,p1)
             ;# MAKE BYTE SHORTWORD
             if dim(3) eq 1 then im=fix(im)
             pim=im(40:dim(1)-41,40:dim(2)-41)
             contrast=[long(min(pim)),long(max(pim))]
             erase
             tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2)
             loadct,0
          endelse
       endelse
    endif
 end
